//////////////////////////////////////////////////////////////////////////////// 
// Copyright (c) 2006 Xilinx, Inc. 
// All Rights Reserved 
//////////////////////////////////////////////////////////////////////////////// 
//   ____  ____ 
//  /   /\/   / 
// /___/  \  /         Vendor: Xilinx 
// \   \   \/          Version: 1.0
//  \   \              Filename: dcr_RAID6.v 
//  /   /              Date Last Modified: 12/23/2005
// /___/   /\          Date Created: 10/6/2005  
// \   \  /  \ 
//  \___\/\___\ 
// 
//Design Name: dcr_RAID6 
//Purpose:  This module implements the DCR registers to allow handshaking between
//          the RAID6 IP and the PPC405
//          
//          
//
////////////////////////////////////////////////////////////////////////////////
// 
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//
//
////////////////////////////////////////////////////////////////////////////////

`timescale 1ns / 100ps

////////////////////////////////////////////////////////////////////////////////
// Module Declaration
////////////////////////////////////////////////////////////////////////////////

module dcr_raid6( 
  // Global Signals
  RST_I,        // I
  CLK_I,        // I
  // DCR Signals 
  DCR_ABus,     // I [0:9] 
  DCR_DBusIn,   // I [0:31]
  DCR_Read,     // I
  DCR_Write,    // I
  DCR_Ack,      // O
  DCR_DBusOut,  // O [0:31]
  RAID_clear,   // I
  RAID_lost,    // O [0:31]
  RAID_reconstruct, // O
  RAID_D1,      // O [0:9]
  RAID_D2,      // O [0:9]
  RAID_D3,      // O [0:9]
  RAID_D4,      // O [0:9]
  RAID_P,       // O [0:9]
  RAID_Q,       // O [0:9]
  RAID_M,       // O [0:9]
  RAID_SIZE,    // O [0:9]
  LED,          // O [0:3]
  GFC_DONE     // I
);

////////////////////////////////////////////////////////////////////////////////
// Port Declarations
////////////////////////////////////////////////////////////////////////////////

  // Global Signals
  input           RST_I;
  input           CLK_I;
  // DCR Signals
  input   [0:9]   DCR_ABus;
  input   [0:31]  DCR_DBusIn;
  input           DCR_Read;
  input           DCR_Write;
  output          DCR_Ack;
  output  [0:31]  DCR_DBusOut;
  input           RAID_clear;
  output  [0:9]  RAID_lost;
  output          RAID_reconstruct;
  output  [0:9]  RAID_D1;
  output  [0:9]  RAID_D2;
  output  [0:9]  RAID_D3;
  output  [0:9]  RAID_D4;
  output  [0:9]  RAID_P;
  output  [0:9]  RAID_Q;
  output  [0:9]  RAID_M;
  output  [0:9]  RAID_SIZE;
  output  [0:3]  LED;
  input          GFC_DONE;
 ////////////////////////////////////////////////////////////////////////////////
// Parameter Declaration
////////////////////////////////////////////////////////////////////////////////

  parameter [0:9]   C_DCR_BASEADDR  = 10'b0000000000;
  parameter [0:9]   C_DCR_HIGHADDR  = 10'b0011111111;

////////////////////////////////////////////////////////////////////////////////
// Signal Declarations
////////////////////////////////////////////////////////////////////////////////
 reg  [0:31] RAID_lost;
 reg         RAID_reconstruct;
 reg [0:31]  RAID_D1;
 reg [0:31]  RAID_D2;
 reg [0:31]  RAID_D3;
 reg [0:31]  RAID_D4;
 reg [0:31]  RAID_P;
 reg [0:31]  RAID_Q;
 reg [0:31]  RAID_M;
 reg [0:31]  RAID_SIZE;
 reg  [0:3]  LED;
 reg         GFC_STATUS;


 wire   [0:31] DCR_DBusIn;
 reg    [0:31] DCR_DBusOutMux;
 reg           DCR_AckReg;

 wire    [0:9] DCR_ABus;
 reg    [0:31] DCR_DBusOut;
 wire          DCR_Write;
 wire          DCR_Read;
 wire          WriteAndDecode, ReadAndDecode;

 reg    [0:11] LowAddrDecode;
 reg           FullAddrDecode;
 reg           WriteAndDecodeReg;
 reg           ReadAndDecodeReg;


////////////////////////////////////////////////////////////////////////////////
// Module Definition
////////////////////////////////////////////////////////////////////////////////

      // Lower Address Decode
     always@( DCR_ABus )
          begin
               case ( DCR_ABus[6:9] )
                    4'b0000 : LowAddrDecode <= 11'b0000_0000_0001;
                    4'b0001 : LowAddrDecode <= 11'b0000_0000_0010;
                    4'b0010 : LowAddrDecode <= 11'b0000_0000_0100;
                    4'b0011 : LowAddrDecode <= 11'b0000_0000_1000;
                    4'b0100 : LowAddrDecode <= 11'b0000_0001_0000;
                    4'b0101 : LowAddrDecode <= 11'b0000_0010_0000;
                    4'b0110 : LowAddrDecode <= 11'b0000_0100_0000;
                    4'b0111 : LowAddrDecode <= 11'b0000_1000_0000;
                    4'b1000 : LowAddrDecode <= 11'b0001_0000_0000;
                    4'b1001 : LowAddrDecode <= 11'b0010_0000_0000;
                    4'b1010 : LowAddrDecode <= 11'b0100_0000_0000;
                    4'b1011 : LowAddrDecode <= 11'b1000_0000_0000;
                    default : LowAddrDecode <= 11'b0000_0000_0000;
               endcase
          end

     // Full Address Decode
     always@( DCR_ABus )
          begin
               case (DCR_ABus[0:9])
                    10'b01_1000_0000 : FullAddrDecode <= 1'b1;
                    10'b01_1000_0001 : FullAddrDecode <= 1'b1;
                    10'b01_1000_0010 : FullAddrDecode <= 1'b1;
                    10'b01_1000_0011 : FullAddrDecode <= 1'b1;
                    10'b01_1000_0100 : FullAddrDecode <= 1'b1;
                    10'b01_1000_0101 : FullAddrDecode <= 1'b1;
                    10'b01_1000_0110 : FullAddrDecode <= 1'b1;
                    10'b01_1000_0111 : FullAddrDecode <= 1'b1;
                    10'b01_1000_1000 : FullAddrDecode <= 1'b1;
                    10'b01_1000_1001 : FullAddrDecode <= 1'b1;
                    10'b01_1000_1010 : FullAddrDecode <= 1'b1;
                    10'b01_1000_1011 : FullAddrDecode <= 1'b1;
                    default          : FullAddrDecode <= 1'b0;
               endcase
          end
     
     // Read and Write AND with Full Decode
     assign WriteAndDecode = FullAddrDecode & DCR_Write;
     assign ReadAndDecode  = FullAddrDecode & DCR_Read;

     // Register Read and Write AND with Full Decode
     always@( posedge CLK_I or posedge RST_I)
          begin
          if ( RST_I ) begin
               WriteAndDecodeReg <= 1'b0;
               ReadAndDecodeReg  <= 1'b0;
          end
          else begin
               WriteAndDecodeReg <= WriteAndDecode;
               ReadAndDecodeReg  <= ReadAndDecode;
          end
     end

     // Generate DCR_Ack for valid Read/Write cycles
     always@( posedge CLK_I or posedge RST_I)
          begin
          if ( RST_I ) begin
               DCR_AckReg <= 1'b0;
          end
          else begin
               DCR_AckReg <=  (WriteAndDecodeReg) |
                                 (ReadAndDecodeReg & ReadAndDecode);
          end
     end

assign DCR_Ack = ( DCR_Write | DCR_Read ) & DCR_AckReg;     

      ////////////////////////////
     //Implement DCR registers
     ////////////////////////////

     // RAID_reconstruct (0x180)
     always@( posedge CLK_I or posedge RST_I)
          begin
          if ( RST_I ) begin
               RAID_reconstruct <= 1'b0;
          end
          else if (( LowAddrDecode == 11'b0000_0000_0001 ) && WriteAndDecode && !DCR_AckReg ) 
          begin
               RAID_reconstruct <= DCR_DBusIn[31];
          end
          else if (RAID_clear)
          begin
               RAID_reconstruct <= 1'b0;
          end
          else
          begin
               RAID_reconstruct <= RAID_reconstruct;
          end
     end
          
     // RAID_lost (0x181)
     always@( posedge CLK_I or posedge RST_I )
          begin
          if ( RST_I )
          begin
               RAID_lost <= 32'b0;
          end
          else if (( LowAddrDecode == 10'b000_0000_0010 ) && WriteAndDecode && !DCR_AckReg ) begin
               RAID_lost <= DCR_DBusIn;
          end
     end

     // RAID_D1 (0x182)
     always@( posedge CLK_I or posedge RST_I )
          begin
          if ( RST_I ) begin
               RAID_D1 <= 32'b0;
         end
          else if (( LowAddrDecode == 11'b0000_0000_0100 ) && WriteAndDecode && !DCR_AckReg ) begin
               RAID_D1 <= DCR_DBusIn;
          end
     end

     // RAID_D2 (0x183)
     always@( posedge CLK_I or posedge RST_I )
          begin
          if ( RST_I ) begin
               RAID_D2 <= 32'b0;
         end
          else if (( LowAddrDecode == 11'b0000_0000_1000 ) && WriteAndDecode && !DCR_AckReg ) begin
               RAID_D2 <= DCR_DBusIn;
          end
     end
      
     
     // RAID_D3 (0x184)
     always@( posedge CLK_I or posedge RST_I )
          begin
          if ( RST_I ) begin
               RAID_D3 <= 32'b0;
         end
          else if (( LowAddrDecode == 11'b0000_0001_0000 ) && WriteAndDecode && !DCR_AckReg ) begin
               RAID_D3 <= DCR_DBusIn;
          end
     end

     // RAID_D4 (0x185)
     always@( posedge CLK_I or posedge RST_I )
          begin
          if ( RST_I ) begin
               RAID_D4 <= 32'b0;
         end
          else if (( LowAddrDecode == 11'b0000_0010_0000 ) && WriteAndDecode && !DCR_AckReg ) begin
               RAID_D4 <= DCR_DBusIn;
          end
     end

    // RAID_P (0x186)
     always@( posedge CLK_I or posedge RST_I )
          begin
          if ( RST_I ) begin
               RAID_P <= 32'b0;
         end
          else if (( LowAddrDecode == 11'b0000_0100_0000 ) && WriteAndDecode && !DCR_AckReg ) begin
               RAID_P <= DCR_DBusIn;
          end
     end

    // RAID_Q (0x187)
     always@( posedge CLK_I or posedge RST_I )
          begin
          if ( RST_I ) begin
               RAID_Q <= 32'b0;
         end
          else if (( LowAddrDecode == 11'b0000_1000_0000 ) && WriteAndDecode && !DCR_AckReg ) begin
               RAID_Q <= DCR_DBusIn;
          end
     end

    // RAID_M (0x188)
     always@( posedge CLK_I or posedge RST_I )
          begin
          if ( RST_I ) begin
               RAID_M <= 32'b0;
         end
          else if (( LowAddrDecode == 11'b0001_0000_0000 ) && WriteAndDecode && !DCR_AckReg ) begin
               RAID_M <= DCR_DBusIn;
          end
     end

    // GFC_STATUS (0x189)
     always@( posedge CLK_I or posedge RST_I )
          begin
          if ( RST_I ) begin
               GFC_STATUS <= 1'b0;
          end
          else if ( GFC_DONE ) begin
               GFC_STATUS <= 1'b1;
          end
          else begin
               GFC_STATUS <= GFC_STATUS;
          end
     end

     // LED (0x18A)
     always@( posedge CLK_I or posedge RST_I )
          begin
          if ( RST_I ) begin
               LED <= 4'b0;
          end          
          else if (( LowAddrDecode == 11'b0100_0000_0000 ) && WriteAndDecode && !DCR_AckReg ) begin
               LED <= DCR_DBusIn[28:31];
          end
     end
     // RAID_SIZE (0x18B)
     always@( posedge CLK_I or posedge RST_I )
          begin
          if ( RST_I ) begin
               RAID_SIZE <= 32'b0;
         end
          else if (( LowAddrDecode == 11'b1000_0000_0000 ) && WriteAndDecode && !DCR_AckReg ) begin
               RAID_SIZE <= DCR_DBusIn;
          end
     end
     //  Register DCR Read Data Bus Selection Mux
     always@( LowAddrDecode)
     begin
          case ( LowAddrDecode ) 
               11'b0000_0000_0001 : DCR_DBusOutMux <= RAID_reconstruct;
               11'b0000_0000_0010 : DCR_DBusOutMux <= RAID_lost;
               11'b0000_0000_0100 : DCR_DBusOutMux <= {21'h000000 , RAID_D1};
               11'b0000_0000_1000 : DCR_DBusOutMux <= {21'h000000 , RAID_D2};
               11'b0000_0001_0000 : DCR_DBusOutMux <= {21'h000000 , RAID_D3};
               11'b0000_0010_0000 : DCR_DBusOutMux <= {21'h000000 , RAID_D4};
               11'b0000_0100_0000 : DCR_DBusOutMux <= {21'h000000 , RAID_P};
               11'b0000_1000_0000 : DCR_DBusOutMux <= {21'h000000 , RAID_Q};
               11'b0001_0000_0000 : DCR_DBusOutMux <= {21'h000000 , RAID_M};
               11'b0010_0000_0000 : DCR_DBusOutMux <= {21'h00000  , GFC_STATUS};
               11'b0100_0000_0000 : DCR_DBusOutMux <= {28'h000000 , LED};
               11'b1000_0000_0000 : DCR_DBusOutMux <= {21'h000000 , RAID_SIZE};
               default      : DCR_DBusOutMux <= 32'b0;
          endcase
     end

     // DCR Bypass Mux
     // DCR ring must either pass incoming data to the read data bus if the
     // address decode is not for this DCR slave, or pass the requested PP
     // DCR register to the DCR_DBusOutMux for reading by the PPC405.
     
     always@( DCR_DBusOutMux or ReadAndDecodeReg or DCR_DBusOut )
     begin
          if ( ReadAndDecodeReg ) begin
               DCR_DBusOut <= DCR_DBusOutMux;
          end
          else begin
               DCR_DBusOut <= DCR_DBusIn;
          end
     end


endmodule
