//////////////////////////////////////////////////////////////////////////////// 
// Copyright (c) 2006 Xilinx, Inc. 
// All Rights Reserved 
//////////////////////////////////////////////////////////////////////////////// 
//   ____  ____ 
//  /   /\/   / 
// /___/  \  /         Vendor: Xilinx 
// \   \   \/          Version: 1.0
//  \   \              Filename: mpmc_if.v 
//  /   /              Date Last Modified: 1/28/2006
// /___/   /\          Date Created: 10/6/2005  
// \   \  /  \ 
//  \___\/\___\ 
// 
//Design Name: mpmc_if 
//Purpose:  This module is the logic that interfaces with the xilinx MPMC block
//          this block can be modified to interface with other type of memory 
//          controller or replaced entirely
//          
//          This is part of XAPP 731
//
////////////////////////////////////////////////////////////////////////////////
// 
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//
//
////////////////////////////////////////////////////////////////////////////////
`timescale 1ns / 1ps

module mpmc_if(
input	 wire        clk,
input  wire        clk_half, 
input  wire        rst,
output reg  [31:0] mp_addr,
output reg         mp_addrreq,
output reg         mp_RNW,
output reg   [1:0] mp_size,
input  wire        mp_addrack,
output reg         mp_rddataack_pos,
output reg         mp_rddataack_neg,
output reg         mp_rdcomp,
output reg         mp_rd_rst,
input  wire [31:0] mp_rddata_pos,
input  wire [31:0] mp_rddata_neg,
input  wire  [4:0] mp_rdwdaddr_pos,
input  wire  [4:0] mp_rdwdaddr_neg,
input  wire        mp_rddatardy,
input  wire        mp_rdfifoempty,
output reg         mp_wrdataack_pos,
output reg         mp_wrdataack_neg,
output wire [31:0] mp_wrdata_pos,
output wire [31:0] mp_wrdata_neg,
output reg         mp_wrcomp,
output reg         mp_wr_rst,
input  wire        mp_wrfifobusy,
input  wire        mp_wrfifofull_pos,
input  wire        mp_wrfifofull_neg,
input  wire        raid_req,
input  wire [31:0] raid_addr,
input  wire        raid_RNW,
input  wire [31:0] raid_wrdata,
input  wire [10:0] sector_len,
output wire        raid_addrack,
output reg         raid_wrrdy,
output reg         raid_dv,
output reg  [31:0] raid_rddata,
output reg         raid_complete
);

reg   [4:0] state;
reg  [10:0] data_len;
reg   [5:0] fifo_wait;
reg  [31:0] wrdata_reg;
wire [31:0] mp_int_addr;
wire        mp_int_RNW;
reg  [31:0] int_addr;
reg         int_addrreq;
reg         int_RNW;
reg   [1:0] int_size; 
wire  [1:0] int_mp_size; 
reg         int_rddataack_pos;
reg         int_rddataack_neg;
wire        int_mp_rddataack_pos;
wire        int_mp_rddataack_neg;
reg         int_rdcomp;
wire        int_mp_rdcomp;
reg         int_rd_rst;
reg         int_wrdataack;
reg  [31:0] int_wrdata;
wire [31:0] int_rddata;
reg         int_wrcomp;
wire        int_mp_wrcomp;
reg         int_wr_rst;
reg         read_fifo_en;
reg         write_fifo_en;
wire        int_rddatardy;
wire        int_mp_rddatardy;
reg   [7:0] delay_cnt;
reg         int_addrack;
wire        int_mp_addrreq;
wire        mp_addrack_reg;
wire        mp_addrreq_reg;
reg         mp_rddataack;
reg         mp_readfifo_write;
reg   [4:0] mp_readfifo_write_count;
reg         mp_wrdataack;
reg         mp_writefifo_read;
reg   [4:0] mp_writefifo_read_count;

parameter IDLE  = 5'b00000;
parameter IDLE1 = 5'b00001;
parameter R0    = 5'b00010;
parameter R1    = 5'b00011;
parameter R2    = 5'b00100;
parameter R3    = 5'b00101;
parameter R4    = 5'b00110;
parameter R5    = 5'b00111;
parameter R6    = 5'b01000;
parameter R7    = 5'b01001;
parameter R8    = 5'b01010;
parameter R9    = 5'b01011;
parameter W0    = 5'b10010;
parameter W1    = 5'b10011;
parameter W2    = 5'b10100;
parameter W3    = 5'b10101;
parameter W4    = 5'b10110;
parameter W5    = 5'b10111;
parameter W6    = 5'b11000;
parameter W7    = 5'b11001;
parameter W8    = 5'b11010;
parameter W9    = 5'b11011;
parameter W10   = 5'b11100;

reg   [3:0] mp_state;
reg   [3:0] mp_datacnt;
wire [31:0] int_mp_addr;

parameter MP_IDLE    = 4'b0000;
parameter MP_START   = 4'b0001;
parameter MP_START2  = 4'b0010;
parameter MP_R0      = 4'b0011;
parameter MP_R1      = 4'b0100;
parameter MP_R2      = 4'b0101;
parameter MP_W0      = 4'b0110;
parameter MP_W1      = 4'b0111;
parameter MP_W2      = 4'b1000;
parameter MP_W3      = 4'b1001;
parameter MP_W4      = 4'b1010;
parameter MP_W5      = 4'b1011;
parameter MP_W6      = 4'b1100;
parameter MP_W7      = 4'b1101;
parameter MP_W8      = 4'b1110;


assign raid_addrack = int_addrack;

//////////////////////////////////////////////
//fifos to allow clock domain crossing

   

wire [26:0] unused_ahf;

handshake_fifo ahf(
.din          ({27'b0,int_addr,int_RNW}),
.rd_clk       (clk_half),
.rd_en        (int_addrack),
.rst          (rst),
.wr_clk       (clk),
.wr_en        (int_addrreq),
.almost_empty (),
.almost_full  (),
.dout         ({unused_ahf,int_mp_addr,int_mp_RNW}),
.empty        (ahf_empty),
.full         ());


async_read_fifo arf(
.din          ({mp_rddata_pos, mp_rddata_neg}),
.rd_clk       (clk),
.rd_en        (read_fifo_en),
.rst          (rst),
.wr_clk       (clk_half),
.wr_en        (mp_rddataack_pos),
.almost_empty (),
.almost_full  (art_full),
.dout         (int_rddata),
.empty        (arf_empty),
.full         ());


async_write_fifo awf(
.din          (int_wrdata),
.rd_clk       (clk_half),
.rd_en        (mp_writefifo_read),//mp_wrdataack_pos),
.rst          (rst),
.wr_clk       (clk),
.wr_en        (write_fifo_en),
.almost_empty (awf_empty),
.almost_full  (awf_full),
.dout         ({mp_wrdata_pos, mp_wrdata_neg}), 
.empty        (),
.full         ());


//this uses a preset and clear to cross clock domains


//assign mp_rddataack_pos = mp_rddataack;
//assign mp_rddataack_neg = mp_rddataack;




always @(posedge clk or posedge rst)
begin
  if(rst)
  begin
    int_addr          <= 32'h00000000;
    int_addrreq       <= 1'b0;
    int_RNW           <= 1'b1;
    mp_rd_rst         <= 1'b0;
    int_wrdataack     <= 1'b0;
    int_wrdata       <= 32'h00000000;
    int_wrcomp        <= 1'b0;
    mp_wr_rst        <= 1'b0;
    raid_wrrdy       <= 1'b0;
    raid_dv          <= 1'b0;
    raid_rddata      <= 32'h00000000;
    raid_complete    <= 1'b0;
    data_len         <= sector_len;
    fifo_wait        <= 5'b00000;
    state            <= IDLE;
    read_fifo_en     <= 1'b0;
    write_fifo_en    <= 1'b0;
  end
  else 
  begin
    case(state)
      IDLE:  begin
               state          <= IDLE1;
               raid_complete  <= 1'b0;
             end
      IDLE1: begin
               if(raid_req)
               begin
                 if(raid_RNW)
                 begin
                   state <= R0;
                 end
                 else
                 begin
                   state    <= W0;
                 end
                 int_addr    <= raid_addr;
                 int_RNW     <= raid_RNW;
                 int_addrreq <= 1'b1;
               end
               else
               begin
                 state <= IDLE; 
               end   
             end
      R0:    begin
               state <= R1;
               int_addrreq <= 1'b0;
             end
      R1:    begin
               if(~arf_empty)
               begin
                 state <= R2;
               end
               else
                 state <= R1;
               end
      R2:    begin
               state      <= R3;
             end
      R3:    begin
               if(data_len == 10'h000)
               begin
                 state             <= R4; 
                 fifo_wait         <= 5'b00000;
               end
               else if(data_len == 10'h00B)
               begin
                 read_fifo_en <= 1'b1;
                 data_len     <= data_len - 1'b1;   //this allowed FIFO to not underflow   
                 state        <= R3;
               end
               else if(data_len <= 10'h00A)
               begin
                 raid_rddata <= int_rddata;
                 data_len    <= data_len - 1'b1;   //Data out of the FIFO should be ready 
                 raid_dv     <= 1'b1;           
                 state       <= R3;
               end
               else
               begin
                 data_len  <= data_len - 1'b1;   //continue            
                 state     <= R3;
               end
                if(raid_dv)
                begin
                  raid_rddata <= int_rddata;
                end              
             end
      R4:    begin
               if(fifo_wait == 5'b10101)
               begin
                 state        <= R5;
                 read_fifo_en <= 1'b0;
                 raid_dv      <= 1'b0;  
               end
               else
               begin
                 state       <= R4;
                 fifo_wait   <= fifo_wait + 1'b1;
                 raid_rddata <= int_rddata;
               end
             end
      R5:    begin
               state         <= R6;
               data_len      <= sector_len;
               fifo_wait     <= 4'b0000;               
             end
      R6:    begin
               state         <= IDLE;
               raid_complete <= 1'b1;
             end
      W0:    begin
               if(int_addrack)
               begin
                 state      <= W1;
               end
               else
               begin
                 state   <= W0;
               end
               int_addrreq <= 1'b0;
             end  
      W1:    begin
               state       <= W2;
               raid_wrrdy  <= 1'b1; 
             end       
      W2:    begin
               state <= W3;
             end
      W3:    begin
               state      <= W4;
               int_wrdata <= raid_wrdata;
               data_len   <= data_len - 1'b1;     
             end
      W4:    begin
               int_wrdata <= raid_wrdata;
               if(data_len == 10'h000)
               begin
                 state      <= W5;
                 fifo_wait  <= 5'b00000;
               end
               else if(data_len == 10'h00F)
               begin
                 state         <= W4;
                 int_wrdataack <= 1'b1;
                 data_len      <= data_len - 1'b1; 
               end
               else
               begin
                 data_len      <= data_len - 1'b1;
                 write_fifo_en <= 1'b1;
                 state         <= W4;
               end
             end
      W5:    begin
                 int_wrdata <= raid_wrdata;
                 if(fifo_wait == 5'b00001)
                 begin
                   write_fifo_en  <= 1'b0; 
                 end
                 if(fifo_wait == 5'b01011)
                 begin
                   state         <= W6;
                   int_wrdataack <= 1'b0;
                   int_wrcomp    <= 1'b1;
                 end
                 else
                 begin
                   state      <= W5;
                   fifo_wait  <= fifo_wait + 1'b1;
                 end              
             end
      W6:    begin
               state         <= W7;
               fifo_wait     <= 4'b0000;
               raid_wrrdy    <= 1'b0; 
             end
      W7:    begin
               if(mp_wrfifofull_pos)
               begin
                 state     <= W7;
               end
               else
               begin
                 state         <= IDLE;
                 raid_complete <= 1'b1;
                 data_len      <= sector_len;
               end
             end
   default:  begin
               state <= IDLE;
               int_addr          <= 32'h00000000;
               int_addrreq       <= 1'b0;
               int_RNW           <= 1'b1;
               int_rd_rst        <= 1'b0;
               int_wrdataack     <= 1'b0;
               int_wrdata        <= 32'h00000000;
               int_wrcomp        <= 1'b0;
               int_wr_rst        <= 1'b0;
               raid_wrrdy        <= 1'b0;
               raid_dv           <= 1'b0;
               raid_rddata       <= 32'h00000000;
               raid_complete     <= 1'b0;
               data_len          <= sector_len;
             end
    endcase 
  end
end

always @(posedge clk_half or posedge rst)
begin
  if(rst)
  begin
    mp_addr           <= 32'h00000000;
    mp_datacnt        <= 4'b1111;
    mp_addrreq        <= 1'b0;
    mp_RNW            <= 1'b0;
    mp_size           <= 2'b00;
    mp_rdcomp         <= 1'b0;
    mp_wrcomp         <= 1'b0;
    mp_rddataack_pos  <= 1'b0;
    mp_rddataack_neg  <= 1'b0;    
    mp_wrdataack_pos  <= 1'b0;
    mp_wrdataack_neg  <= 1'b0;
    mp_state          <= MP_IDLE;
    int_addrack       <= 1'b0;
    mp_writefifo_read <= 1'b0;
  end
  else 
  begin
    case(mp_state)
   MP_IDLE:  begin
               if(~ahf_empty)
               begin
                 int_addrack <= 1'b1;
                 mp_state    <= MP_START;
               end
               else
               begin
                 mp_state <= MP_IDLE;
               end
             end
 MP_START:   begin
               int_addrack <= 1'b0;
               mp_addr     <= int_mp_addr;
               mp_RNW      <= int_mp_RNW;
               mp_size     <= 2'b11;
               mp_addrreq  <= 1'b1; 
               mp_state    <= MP_START2;
             end
  MP_START2: begin
               if(mp_addrack)
               begin                
                 mp_size    <= 2'b00;
                 mp_addrreq <= 1'b0;
                 if(mp_RNW)
                 begin
                   mp_state  <= MP_R0; 
                 end
                 else
                 begin
                   mp_state  <= MP_W0; 
                 end
               end
               else
               begin
                 mp_state   <= MP_START2;
               end
             end
    MP_R0:   begin
               if(mp_rddatardy)
               begin
                 mp_rddataack_pos <= 1'b1;
                 mp_rddataack_neg <= 1'b1;
                 mp_state         <= MP_R1;
               end
               else
               begin
                 mp_state <= MP_R0;
               end
             end
    MP_R1:   begin
               if(mp_datacnt == 4'b0000)
               begin
                 mp_datacnt       <= 4'b1111;
                 mp_state         <= MP_R2;
                 mp_rddataack_pos <= 1'b0;
                 mp_rddataack_neg <= 1'b0;
                 mp_rdcomp        <= 1'b1;
               end
               else
               begin
                 mp_datacnt  <= mp_datacnt - 1'b1;
                 mp_state    <= MP_R1;
               end
             end
    MP_R2:   begin
               mp_state  <= MP_IDLE;
               mp_rdcomp <= 1'b0;
             end
    MP_W0:   begin
               if(~awf_empty)
               begin
                 mp_state <= MP_W1;
               end
               else
               begin
                 mp_state <= MP_W0;
               end
             end
    MP_W1:   begin
               mp_state <= MP_W2;
             end
    MP_W2:   begin
               mp_state <= MP_W3;
             end

    MP_W3:   begin
               mp_state <= MP_W4;
             end
    MP_W4:   begin
               mp_state <= MP_W5;
             end
    MP_W5:   begin
               if(mp_datacnt == 4'b0000)
               begin
                 mp_datacnt <= 4'b1111;
                 mp_state   <= MP_W6;
               end
               else if(mp_datacnt == 4'b0111)
               begin
                 mp_datacnt  <= mp_datacnt - 1'b1;                 
                 mp_state    <= MP_W5;
               end
               else if(mp_datacnt == 4'b1111)
               begin
                 mp_datacnt        <= mp_datacnt - 1'b1;
                 mp_wrdataack_pos  <= 1'b1;
                 mp_wrdataack_neg  <= 1'b1;
                 mp_writefifo_read <= 1'b1;
                 mp_state          <= MP_W5;
               end
               else
               begin
                 mp_datacnt  <= mp_datacnt - 1'b1;
                 mp_state    <= MP_W5;
               end
             end
    MP_W6:   begin
               mp_state          <= MP_W7;
               mp_writefifo_read <= 1'b0;
               mp_wrdataack_pos  <= 1'b0;
               mp_wrdataack_neg  <= 1'b0;
               mp_wrcomp         <= 1'b1;
             end
    MP_W7:   begin
               mp_state  <= MP_IDLE;
               mp_wrcomp <= 1'b0;
             end
    default: begin
               mp_state  <= MP_IDLE;
             end
    endcase
  end
end

endmodule
